package com.guwan.backend.service;

import lombok.RequiredArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.apache.hadoop.fs.FSDataInputStream;
import org.apache.hadoop.fs.FSDataOutputStream;
import org.apache.hadoop.fs.FileSystem;
import org.apache.hadoop.fs.Path;
import org.springframework.stereotype.Service;
import org.springframework.web.multipart.MultipartFile;

import java.io.BufferedInputStream;
import java.io.IOException;
import java.io.InputStream;

@Slf4j
@Service
@RequiredArgsConstructor
public class HdfsStorageService {

    private final FileSystem fileSystem;
    private static final String VIDEO_PATH = "/videos";
    private static final int BUFFER_SIZE = 4096;

    /**
     * 上传视频到HDFS
     */
    public String uploadVideo(MultipartFile file, String fileName) throws IOException {
        String hdfsPath = VIDEO_PATH + "/" + fileName;
        Path path = new Path(hdfsPath);

        try (FSDataOutputStream out = fileSystem.create(path);
             BufferedInputStream in = new BufferedInputStream(file.getInputStream())) {
            
            byte[] buffer = new byte[BUFFER_SIZE];
            int length;
            while ((length = in.read(buffer)) > 0) {
                out.write(buffer, 0, length);
            }
            
            return hdfsPath;
        }
    }

    /**
     * 从HDFS读取视频流
     */
    public InputStream getVideoStream(String path) throws IOException {
        Path hdfsPath = new Path(path);
        if (!fileSystem.exists(hdfsPath)) {
            throw new IOException("Video not found: " + path);
        }
        return fileSystem.open(hdfsPath);
    }

    /**
     * 删除HDFS中的视频
     */
    public void deleteVideo(String path) throws IOException {
        Path hdfsPath = new Path(path);
        if (fileSystem.exists(hdfsPath)) {
            fileSystem.delete(hdfsPath, false);
        }
    }

    /**
     * 获取视频元数据
     */
    public VideoMetadata getVideoMetadata(String path) throws IOException {
        Path hdfsPath = new Path(path);
        if (!fileSystem.exists(hdfsPath)) {
            throw new IOException("Video not found: " + path);
        }

        return VideoMetadata.builder()
            .size(fileSystem.getFileStatus(hdfsPath).getLen())
            .blockSize(fileSystem.getFileStatus(hdfsPath).getBlockSize())
            .replication(fileSystem.getFileStatus(hdfsPath).getReplication())
            .modificationTime(fileSystem.getFileStatus(hdfsPath).getModificationTime())
            .build();
    }
} 